mod adapt;
mod allow_unvalidated;
mod alphanumeric;
mod ascii;
mod contains;
mod credit_card;
mod custom;
mod dive;
mod dive_with_ctx;
mod dive_with_rules;
mod email;
mod inner;
mod ip;
mod length;
mod matches;
mod multi_rule;
mod newtype;
mod option;
mod pattern;
mod phone_number;
mod prefix;
mod range;
mod select;
mod skip;
mod suffix;
mod url;

mod util;
